/*  This program is a part of the 'T_H_X10' project, found at
       http://members.home.net/mkeryan/t_h_x10/

    It is Copyrighted by under the GPL:  (c) 1999 Michael J. Keryan
    See the file "COPYING" included with this program.
    You are granted permission to use and modify the code with the
    stipulation that copyright notices remain and the author is to be
    notified of any improvements, bug fixes, etc.

    graph_th.c  inputs data put out hourly from t_h_x10; it consists of
                average Temperature and Humidity of each channel with hourly
		count, and the floor averages (when > 1 / floor), where the
		count refers now to # of channels in the average. A couple of
		sample lines:
		   Hr= 18 F07  T= 74 n= 3  H= 41 n= 4
		   Hr= 18 Z01  T= 74 n= 2  H= 43 n= 3
		This program works on this data and creates a graph using
		gnuplot and converts it to a gif.
*/

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#define DIRPATH "/usr/local/t_h_x10/"
#define TMPPATH "/tmp/"
#define BUFSIZE 256

int main(int argc, char* argv[])
{
  FILE *fileptr_avg = NULL;
  FILE *fileptr_t[8] = {NULL};
  FILE *fileptr_h[8] = {NULL};
  FILE *fileptr_tz[4] = {NULL};
  FILE *fileptr_th[4] = {NULL};
  struct tm *timest;
  time_t now;
  char datestr[BUFSIZE];
  char filename[BUFSIZE], filename1[BUFSIZE], filename2[BUFSIZE];
  char aline[BUFSIZE];
  char dummy1[BUFSIZE], dummy2[BUFSIZE], dummy3[BUFSIZE], dummy4[BUFSIZE],
    dummy5[BUFSIZE];
  char type[BUFSIZE];
  const char *anum[] = {"1","2","3","4","5","6","7","8"};
  const char *bnum[] = {"0","1","2","3"};
  int i, numargs, hr, number, T, H, nt, nh;

  /* get todays date */
  now = time(NULL);
  if ((timest = localtime(&now)) == NULL) {
    fprintf(stderr, "Cannot find date, aborting!\n");
    exit(-1);
  }

  strftime(datestr,BUFSIZE-1,"%Y%m%d",timest);

  /* open the data file */
  strcpy(filename,DIRPATH);
  strcat(filename,"data/avg_");
  strcat(filename,datestr);
  strcat(filename,".log");
  if((fileptr_avg = fopen(filename,"r")) == NULL) {
    fprintf(stderr, "Cannot open data file: %s, aborting!\n", filename);
    exit(-1);
  }

  /* open up 24 temp. files for different channels, etc. */
  strcpy(filename,TMPPATH);
  strcpy(filename1,filename);
  strcat(filename1,"T-");
  strcpy(filename2,filename);
  strcat(filename2,"H-");

  for (i=0; i < 8; i++) {
    strcpy(filename,filename1);
    strcat(filename,anum[i]);
    if ((fileptr_t[i] = fopen(filename,"w")) == NULL) {
      fprintf(stderr, "Cannot open temp file %s; aborting!\n", filename);
      exit(-1);
    }
    strcpy(filename,filename2);
    strcat(filename,anum[i]);
    if ((fileptr_h[i] = fopen(filename,"w")) == NULL) {
      fprintf(stderr, "Cannot open temp file %s; aborting!\n", filename);
      exit(-1);
    }
  }
  
  /* above 16 were t, h data; 8 more for avg of floors */
  strcpy(filename,TMPPATH);
  strcpy(filename1,filename);
  strcat(filename1,"TF");
  strcpy(filename2,filename);
  strcat(filename2,"HF");
  for (i=0; i < 4; i++) {
    strcpy(filename,filename1);
    strcat(filename,bnum[i]);
    if ((fileptr_tz[i] = fopen(filename,"w")) == NULL) {
      fprintf(stderr, "Cannot open temp file %s; aborting!\n", filename);
      exit(-1);
    }
    strcpy(filename,filename2);
    strcat(filename,bnum[i]);
    if ((fileptr_th[i] = fopen(filename,"w")) == NULL) {
      fprintf(stderr, "Cannot open temp file %s; aborting!\n", filename);
      exit(-1);
    }
  }
  
  /* now go through each line of data and write to appropriate file */
  while (fgets(aline, sizeof(aline), fileptr_avg) != NULL) {
    if ((numargs = sscanf(aline, "%s %d %1s%2d %s %d %s %d %s %d %s %d",
			  dummy1, &hr, type, &number, dummy2, &T, dummy3, &nt,
			  dummy4, &H, dummy5, &nh)) > 11) {
      printf(" %d %d %d %d %s%d\n", T, nt, H, nh, type, number);
      if (!strcmp(type, "Z")) {
	if (nt > 0) {
	  fprintf(fileptr_tz[number], "%2d %3d\n", hr, T);
	}
	if (nh > 0) {
	  fprintf(fileptr_th[number], "%2d %3d\n", hr, H);
	}
      } else {
	if (nt > 0) {
	  fprintf(fileptr_t[number - 1], "%2d %3d\n", hr, T);
	}
	if (nh > 0) {
	  fprintf(fileptr_h[number - 1], "%2d %3d\n", hr, H);
	}
      }
    }
  }
  return 0;
}

